#cs ----------------------------------------------------------------------------

 AutoIt Version: 3.3.16.1
 Author:         Kanashius

 Script Function:
	Generate QRCode over the command line interface (cli).

#ce ----------------------------------------------------------------------------
#pragma compile(Console, true)
#include "QRCode.au3"

_QRCode_StartUp()
If @error Then
	ConsoleWrite("QRCode startup failed: "&@error&@crlf)
	If @error=1 Then ConsoleWrite("Check if the required """&$_QRCode_DLL_FileName&'" is in the same folder as this program!'&@crlf)
EndIf

; parse commandline and set variables accordingly
Global $bShowHelp = False, $sText = Default, $sDestPath = Default, $iECL = $_QRCode_ECL_HIGH, $color = "#", $bg = " ", $sLineBreak = @crlf, $sIconPath = Default, $iSize = Default, $arBorder=[0, 0, 0, 0]
_parseCommandLine()

_process()

_exit()

Func _process()
	Local $arQRCode = _QRCode_GetQRCode($sText, $iECL)
	If @error Then
		If @error=6 Then
			ConsoleWrite("QRCode generation failed, because the input text is to large. Make it smaller or try a smaller error correction level (ECL)"&@crlf)
			_exit(12)
		EndIf
		ConsoleWrite("QRCode generation failed: "&@error&" >> "&@extended&@crlf)
		_exit(11)
	EndIf
	If $sDestPath=Default Then
		If $iSize=Default Then $iSize = 2
		_QRCode_ConsoleWrite($arQrCode, $color, $bg, $sLineBreak, $iSize, $arBorder[0], $arBorder[1], $arBorder[2], $arBorder[3])
		If @error Then
			ConsoleWrite("Error creating the console output of the qrcode: "&@error&" >> "&@extended&@crlf)
			_exit(15)
		EndIf
	Else
		If $iSize=Default Then $iSize = 1
		Local $iColor = Int($color)
		Local $iBg = Int($bg)
		Local $hBitmap = _QRCode_GetBitmap($arQrCode, $iSize, $arBorder[0], $arBorder[1], $arBorder[2], $arBorder[3], $iColor, $iBg)
		If @error Then
			ConsoleWrite("Error creating the image of the qrcode: "&@error&" >> "&@extended&@crlf)
			_exit(15)
		EndIf
		If $sIconPath<>Default Then
			Local $hBitmapIcon = _GDIPlus_BitmapCreateFromFile($sIconPath)
			If @error Then
				ConsoleWrite("Error loading the icon image file from "&'"'&$sIconPath&'": '&@error&" >> "&@extended&@crlf)
				_exit(14)
			EndIf
			Local $iLossFactor = _QRCode_GetECLMaxLoss($iECL)-0.02 ; Subtract 0.02 (2%) to be save and still allow errors reading the image
			Local $iQRCodeSize = UBound($arQRCode)*$iSize
			Local $iIconSize = Int($iQRCodeSize*$iLossFactor)
			Local $iIconWidth = _GDIPlus_ImageGetWidth($hBitmapIcon)
			Local $iIconHeight = _GDIPlus_ImageGetHeight($hBitmapIcon)
			Local $iFactorWidth = $iIconSize/$iIconWidth
			Local $iFactorHeight = $iIconSize/$iIconHeight
			Local $iFactor = $iFactorWidth<$iFactorHeight?$iFactorWidth:$iFactorHeight
			If $iFactor>1 Then $iFactor=1 ; Do not scale icons up; only downscaling if to large
			Local $hBitmapIconResized = _GDIPlus_ImageResize($hBitmapIcon, $iIconWidth*$iFactor, $iIconHeight*$iFactor)
			_GDIPlus_ImageDispose($hBitmapIcon)
			Local $hGraphics = _GDIPlus_ImageGetGraphicsContext($hBitmap)
			$iIconHeight = _GDIPlus_ImageGetHeight($hBitmapIconResized)
			$iIconWidth = _GDIPlus_ImageGetWidth($hBitmapIconResized)
			Local $iLeft = $iQRCodeSize/2-$iIconSize/2+($iIconSize-$iIconWidth)/2
			Local $iTop = $iQRCodeSize/2-$iIconSize/2+($iIconSize-$iIconHeight)/2
			_GDIPlus_GraphicsDrawImageRect($hGraphics, $hBitmapIconResized, $iLeft, $iTop, $iIconWidth, $iIconHeight)
			_GDIPlus_GraphicsDispose($hGraphics)
			_GDIPlus_ImageDispose($hBitmapIconResized)
		EndIf
		_GDIPlus_ImageSaveToFile($hBitmap, $sDestPath)
		If @error Then
			ConsoleWrite("Error saving the QRCode image file to "&'"'&$sDestPath&'": '&@error&" >> "&@extended&@crlf)
			_exit(13)
		EndIf
	EndIf
EndFunc

Func _exit($iCode=0)
	If $bShowHelp or $iCode<>0 or UBound($CmdLine)=1 Then _showHelp()
	_QRCode_Shutdown()
	Exit $iCode
EndFunc

Func _parseCommandLine()
	For $i=1 To UBound($CmdLine)-1 Step 1
		Switch $CmdLine[$i]
			Case "/?", "help", "-h", "--help", "?"
				$bShowHelp = True
			Case "--text", "-t"
				$i+=1
				If $i>UBound($CmdLine)-1 Then
					ConsoleWrite('Text path value is missing!'&@crlf)
					_exit(9)
				EndIf
				$sText = $CmdLine[$i]
			Case "--out", "-o"
				$i+=1
				If $i>UBound($CmdLine)-1 Then
					ConsoleWrite('Destination value is missing!'&@crlf)
					_exit(1)
				EndIf
				$sDestPath = $CmdLine[$i]
			Case "--ecl", "-e"
				$i+=1
				If $i>UBound($CmdLine)-1 Then
					ConsoleWrite('ECL value is missing!'&@crlf)
					_exit(2)
				EndIf
				$iECL = Int($CmdLine[$i])
				If $iECL<1 or $iECL>4 Then
					ConsoleWrite('ECL of "'&$CmdLine[$i]&'" is not allowed! Must be one of [1,2,3,4]'&@crlf)
					_exit(2)
				EndIf
			Case "--color", "-c"
				$i+=1
				If $i>UBound($CmdLine)-1 Then
					ConsoleWrite('Color value is missing!'&@crlf)
					_exit(3)
				EndIf
				$color = $CmdLine[$i]
			Case "--background", "-b"
				$i+=1
				If $i>UBound($CmdLine)-1 Then
					ConsoleWrite('Background value is missing!'&@crlf)
					_exit(4)
				EndIf
				$bg = $CmdLine[$i]
			Case "--linebreak", "-l"
				$i+=1
				If $i>UBound($CmdLine)-1 Then
					ConsoleWrite('Linebreak value is missing!'&@crlf)
					_exit(5)
				EndIf
				$sLineBreak = $CmdLine[$i]
			Case "--icon", "-i"
				$i+=1
				If $i>UBound($CmdLine)-1 Then
					ConsoleWrite('Icon value is missing!'&@crlf)
					_exit(6)
				EndIf
				$sIconPath = $CmdLine[$i]
				If Not FileExists($sIconPath) Then
					ConsoleWrite('Icon file does not exist: "'&$sIconPath&'"'&@crlf)
					_exit(6)
				EndIf
			Case "--size", "-s"
				$i+=1
				If $i>UBound($CmdLine)-1 Then
					ConsoleWrite('Size value is missing!'&@crlf)
					_exit(7)
				EndIf
				$iSize = Int($CmdLine[$i])
				If $iSize<1 Then
					ConsoleWrite('ECL of "'&$CmdLine[$i]&'" is not allowed! Must be a positive integer >0!'&@crlf)
					_exit(7)
				EndIf
			Case "--margin", "-m"
				$i+=1
				If $i>UBound($CmdLine)-1 Then
					ConsoleWrite('Margin value is missing!'&@crlf)
					_exit(8)
				EndIf
				$arBorder = StringRegExp($CmdLine[$i], "\s*\[\s*(\d+)\s*,\s*(\d+)\s*,\s*(\d+)\s*,\s*(\d+)\s*\]\s*", 3)
				If @error Then
					ConsoleWrite('Margin "'&$CmdLine[$i]&'" could not be parsed! Must be an array of integer values like "[0,0,0,0]"!'&@crlf)
					_exit(8)
				EndIf
				$arBorder[0] = Int($arBorder[0])
				$arBorder[1] = Int($arBorder[1])
				$arBorder[2] = Int($arBorder[2])
				$arBorder[3] = Int($arBorder[3])
			Case Else
				If $sText=Default Then
					$sText = $CmdLine[$i]
				ElseIf $sDestPath=Default Then
					$sDestPath = $CmdLine[$i]
				Else
					ConsoleWrite("Unknown parameter: "&$CmdLine[$i]&@crlf)
					_exit(10)
				EndIf
		EndSwitch
	Next
	If $sText=Default Then
		ConsoleWrite("Missing input text!"&@crlf)
		_exit(1)
	EndIf
EndFunc

Func _showHelp()
	ConsoleWrite("================================ QRCode generator - cli ================================"&@crlf)
	ConsoleWrite("The QRCode generator cli provides a command line interface to generate qrcodes."&@crlf)
	ConsoleWrite("It allows the user to print the qrcode to the console (if no destination path"&@crlf)
	ConsoleWrite("is provided) or to output the qrcode as an image. It also allows the addition"&@crlf)
	ConsoleWrite("of another image to be drawn ontop of the qrcode in the middle, with the size"&@crlf)
	ConsoleWrite("automatically adjusted accordingly to the error correction level (ECL)."&@crlf)
	ConsoleWrite(@crlf)
	ConsoleWrite(@crlf)
	ConsoleWrite("SYNOPSIS"&@crlf)
	ConsoleWrite(@crlf)
	ConsoleWrite('qrcodegenerator-cli [OPTION]... [INPUT TEXT] [OPTION]... [DESTINATION] [OPTION]...'&@crlf)
	ConsoleWrite(@crlf)
	ConsoleWrite(@crlf)
	ConsoleWrite("DESCRIPTION"&@crlf)
	ConsoleWrite(@crlf)
	ConsoleWrite("Generate a qrcode for the given input text."&@crlf)
	ConsoleWrite("The qrcode is printed to the console, or if a DESTINATION is provided, the output is"&@crlf)
	ConsoleWrite("saved as an image to that DESTINATION."&@crlf)
	ConsoleWrite("The INPUT TEXT is optional, but then it must be replaced with the flag -t/--text."&@crlf)
	ConsoleWrite("The DESTINATION can be replaced with the flag -d/--out."&@crlf)
	ConsoleWrite(@crlf)
	ConsoleWrite("-b, --background"&@crlf)
	ConsoleWrite(@TAB&'provide the desired value for the background.'&@crlf)
	ConsoleWrite(@TAB&'Image output: color as ARGB (Default "0xFFFFFFFF").'&@crlf)
	ConsoleWrite(@TAB&'Console output: the desired character/string (Default " ").'&@crlf)
	ConsoleWrite("-c, --color"&@crlf)
	ConsoleWrite(@TAB&'provide the desired value for the foreground.'&@crlf)
	ConsoleWrite(@TAB&'Image output: color as ARGB (Default "0xFF000000").'&@crlf)
	ConsoleWrite(@TAB&'Console output: the desired character/string (Default "#").'&@crlf)
	ConsoleWrite("-e, --ecl"&@crlf)
	ConsoleWrite(@TAB&'provide the error correction level (ECL) (Default "4").'&@crlf)
	ConsoleWrite(@TAB&'Possible values:'&@crlf)
	ConsoleWrite(@TAB&'Low (up to ~7% loss): 1'&@crlf)
	ConsoleWrite(@TAB&'Medium (up to ~15% loss): 2'&@crlf)
	ConsoleWrite(@TAB&'Quartile (up to ~25% loss): 3'&@crlf)
	ConsoleWrite(@TAB&'High (up to ~30% loss): 4'&@crlf)
	ConsoleWrite("-h, --help, help, /?, ?"&@crlf)
	ConsoleWrite(@TAB&'Show this help'&@crlf)
	ConsoleWrite("-i, --icon"&@crlf)
	ConsoleWrite(@TAB&'provide the path to an image. That image will then be drawn at the qrcode.'&@crlf)
	ConsoleWrite(@TAB&'It will be scaled to not be larger then the allowed ECL (see argument'&@crlf)
	ConsoleWrite(@TAB&'above). Image size in relation to the qrcode and with those ECLs:'&@crlf)
	ConsoleWrite(@TAB&'low 5%, medium 13%, quartile 23%, high 28%.'&@crlf)
	ConsoleWrite(@TAB&'Note: The icon will not be upscaled, if it is smaller.'&@crlf)
	ConsoleWrite(@TAB&'Note: You can change -s/--size to make the qrcode larger => more imagespace.'&@crlf)
	ConsoleWrite("-l, --linebreak"&@crlf)
	ConsoleWrite(@TAB&'provide the desired character/string to insert after every qrcode line'&@crlf)
	ConsoleWrite(@TAB&'(Default "\r\n" or "@CRLF"). This only works with console output.'&@crlf)
	ConsoleWrite("-m, --margin"&@crlf)
	ConsoleWrite(@TAB&'configure a border around the qrcode (Default "[0,0,0,0]").'&@crlf)
	ConsoleWrite(@TAB&'Defined as integer values: [<Top>,<Right>,<Bottom>,<Left>]'&@crlf)
	ConsoleWrite(@TAB&'With image output, it defines the border in pixels.'&@crlf)
	ConsoleWrite(@TAB&'With console output, it defines the border above/below with "\r\n" (@CRLF) and'&@crlf)
	ConsoleWrite(@TAB&'to the left/right with spaces (" ").'&@crlf)
	ConsoleWrite("-s, --size"&@crlf)
	ConsoleWrite(@TAB&'provide the desired size for the output.'&@crlf)
	ConsoleWrite(@TAB&'Image output: Pixel per rectangle'&@crlf)
	ConsoleWrite(@TAB&'Console output: Character/String per rectangle'&@crlf)
	ConsoleWrite(@TAB&'NOTE (Image output): Increasing the size changes the output width/height. This'&@crlf)
	ConsoleWrite(@TAB&'also influences the size of a possible icon (-i/--icon).'&@crlf)
	ConsoleWrite("-t, --text"&@crlf)
	ConsoleWrite(@TAB&'provides an alternative to define the input text. This allows the user to be'&@crlf)
	ConsoleWrite(@TAB&'sure, the text is not misinterpreted as another argument.'&@crlf)
	ConsoleWrite("-o, --out"&@crlf)
	ConsoleWrite(@TAB&'provides an alternative to define the destination. This allows the user to be'&@crlf)
	ConsoleWrite(@TAB&'sure the destination is not misinterpreted as another argument.'&@crlf)
	ConsoleWrite(@crlf)
	ConsoleWrite(@crlf)
	ConsoleWrite("ERROR HANDLING"&@crlf)
	ConsoleWrite(@crlf)
	ConsoleWrite("If an error occurres, the program prints an error and exits with an error code."&@crlf)
	ConsoleWrite('Exit codes:'&@crlf)
	ConsoleWrite(@TAB&'1  - Missing arguments; at least the input text is required'&@crlf)
	ConsoleWrite(@TAB&'2  - Parameter --ecl provided, but value is missing or not one of [1,2,3,4]'&@crlf)
	ConsoleWrite(@TAB&'3  - Parameter --color provided, but value is missing'&@crlf)
	ConsoleWrite(@TAB&'4  - Parameter --background provided, but value is missing'&@crlf)
	ConsoleWrite(@TAB&'5  - Parameter --linebreak provided, but value is missing'&@crlf)
	ConsoleWrite(@TAB&'6  - Parameter --icon provided, but value is missing or path does not exist'&@crlf)
	ConsoleWrite(@TAB&'7  - Parameter --size provided, but value is missing or not an allowed value'&@crlf)
	ConsoleWrite(@TAB&'8  - Parameter --margin provided, but value is missing or not an allowed value'&@crlf)
	ConsoleWrite(@TAB&'9  - Parameter --out provided, but value is missing'&@crlf)
	ConsoleWrite(@TAB&'10 - Unknown parameter found'&@crlf)
	ConsoleWrite(@TAB&'11 - QRCode generation failed.'&@crlf)
	ConsoleWrite(@TAB&'12 - QRCode generation failed because of the input text being to large.'&@crlf)
	ConsoleWrite(@TAB&'     Maybe try a lower error correction level (ECL).'&@crlf)
	ConsoleWrite(@TAB&'13 - QRCode image file could not be saved. There may be a problem with the path.'&@crlf)
	ConsoleWrite(@TAB&'14 - Icon image file could not be loaded as an image'&@crlf)
	ConsoleWrite(@TAB&'15 - Drawing (image)/showing (console) the qrcode failed'&@crlf)
	ConsoleWrite(@crlf)
	ConsoleWrite(@crlf)
	ConsoleWrite("AUTHOR"&@crlf)
	ConsoleWrite(@crlf)
	ConsoleWrite('Kanashius'&@crlf)
	ConsoleWrite('Copyright © 2024 Kanashius (https://wwww.kanashius.com)'&@crlf)
EndFunc